@@ -181,6 +181,14 @@ pub enum GraphError {
         estimated_completion: Option<u64>,
     },
 
+    /// Invalid state for the requested operation
+    InvalidState {
+        operation: String,
+        expected_state: String,
+        actual_state: String,
+        suggestion: String,
+    },
+
     /// Lock acquisition failed
     LockContentionError { resource: String, timeout_ms: u64 },
 
@@ -209,6 +217,13 @@ pub enum GraphError {
         detected_format: Option<String>,
     },
 
+    /// Failed to convert between different data types or formats
+    ConversionError {
+        from: String,
+        to: String,
+        details: String,
+    },
+
     /*
     === QUERY ERRORS ===
     Problems with filtering and query operations
@@ -236,6 +251,9 @@ pub enum GraphError {
         partial_results: Option<usize>,
     },
 
+    /// Query parsing failed
+    QueryParseError { query: String, message: String },
+
     /*
     === CONFIGURATION ERRORS ===
     Problems with settings and configuration
@@ -280,6 +298,17 @@ pub enum GraphError {
         actual: String,
         operation: String,
     },
+
+    /*
+    === VISUALIZATION/LAYOUT ERRORS ===
+    Problems with layout computation and visualization
+    */
+    /// Layout computation failed
+    LayoutError {
+        operation: String,
+        layout_type: String,
+        error_details: String,
+    },
 }
 
 impl GraphError {
@@ -429,7 +458,8 @@ impl GraphError {
             | GraphError::UncommittedChanges { .. }
             | GraphError::AttributeNotFound { .. }
             | GraphError::NoChangesToCommit
-            | GraphError::InvalidQuery { .. } => true,
+            | GraphError::InvalidQuery { .. }
+            | GraphError::LayoutError { .. } => true,
 
             // Definitely not recoverable - system is in bad state
             GraphError::CorruptedHistory { .. }
@@ -477,9 +507,11 @@ impl GraphError {
                 ErrorCategory::Query
             }
 
-            GraphError::OperationInProgress { .. } | GraphError::LockContentionError { .. } => {
-                ErrorCategory::Concurrency
-            }
+            GraphError::OperationInProgress { .. }
+            | GraphError::LockContentionError { .. }
+            | GraphError::InvalidState { .. } => ErrorCategory::Concurrency,
+
+            GraphError::LayoutError { .. } => ErrorCategory::Visualization,
 
             GraphError::InternalError { .. }
             | GraphError::UnexpectedState { .. }
@@ -560,6 +592,7 @@ pub enum ErrorCategory {
     Persistence,
     Query,
     Concurrency,
+    Visualization,
     Internal,
     Other,
 }
@@ -800,3 +833,14 @@ TESTING STRATEGY:
 - Test error formatting for user-friendliness
 - Test that internal errors include sufficient debugging context
 */
+
+// Error type conversions
+impl From<crate::storage::advanced_matrix::unified_matrix::MatrixError> for GraphError {
+    fn from(error: crate::storage::advanced_matrix::unified_matrix::MatrixError) -> Self {
+        GraphError::ConversionError {
+            from: "MatrixError".to_string(),
+            to: "GraphError".to_string(),
+            details: format!("{:?}", error),
+        }
+    }
+}
