@@ -28,6 +28,64 @@ pub type StateId = u64;
 /// Branch name for git-like workflow
 pub type BranchName = String;
 
+/// Subgraph identifier for entity system
+pub type SubgraphId = usize;
+
+/// Universal entity identifier for the GraphEntity trait system
+#[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
+pub enum EntityId {
+    /// Node entity
+    Node(NodeId),
+    /// Edge entity
+    Edge(EdgeId),
+    /// Subgraph entity (base subgraph)
+    Subgraph(SubgraphId),
+    /// Neighborhood subgraph entity
+    Neighborhood(SubgraphId),
+    /// Component subgraph entity
+    Component(SubgraphId),
+    /// Path subgraph entity
+    Path(SubgraphId),
+    /// Filter subgraph entity
+    Filter(SubgraphId),
+    /// Meta-node entity (node containing a subgraph)
+    MetaNode(NodeId),
+}
+
+impl EntityId {
+    /// Check if this EntityId represents a valid identifier
+    pub fn is_valid(&self) -> bool {
+        match self {
+            EntityId::Node(id) | EntityId::MetaNode(id) => *id != usize::MAX,
+            EntityId::Edge(id) => *id != usize::MAX,
+            EntityId::Subgraph(id)
+            | EntityId::Neighborhood(id)
+            | EntityId::Component(id)
+            | EntityId::Path(id)
+            | EntityId::Filter(id) => *id != usize::MAX,
+        }
+    }
+
+    /// Get the underlying numeric ID regardless of entity type
+    pub fn numeric_id(&self) -> usize {
+        match self {
+            EntityId::Node(id) | EntityId::MetaNode(id) => *id,
+            EntityId::Edge(id) => *id,
+            EntityId::Subgraph(id)
+            | EntityId::Neighborhood(id)
+            | EntityId::Component(id)
+            | EntityId::Path(id)
+            | EntityId::Filter(id) => *id,
+        }
+    }
+}
+
+impl Default for EntityId {
+    fn default() -> Self {
+        EntityId::Node(0)
+    }
+}
+
 /*
 === GRAPH STRUCTURE TYPES ===
 Fundamental graph properties that affect behavior across the entire system.
@@ -63,7 +121,7 @@ This is the heart of the flexible attribute system. Should support:
 
 /// Efficient storage for attribute values supporting multiple data types
 /// DESIGN: Enum dispatch is fast, Hash implementation handles f32 properly
-#[derive(Debug, Clone, serde::Serialize)]
+#[derive(Debug, Clone, serde::Serialize, serde::Deserialize)]
 pub enum AttrValue {
     /// 32-bit float (embeddings, coordinates, ML features)
     Float(f32),
@@ -74,6 +132,12 @@ pub enum AttrValue {
     /// Vector of floats (embeddings, coordinates, feature vectors)
     /// PERFORMANCE: Vec<f32> is more cache-friendly than Vec<AttrValue>
     FloatVec(Vec<f32>),
+    /// Vector of integers (IDs, counts, indices)
+    IntVec(Vec<i64>),
+    /// Vector of strings (tags, labels, categories)
+    TextVec(Vec<String>),
+    /// Vector of booleans (flags, masks, selections)
+    BoolVec(Vec<bool>),
     /// Boolean flag (active, enabled, etc.)
     Bool(bool),
     /// Memory-optimized compact string for short text values (Memory Optimization 1)
@@ -89,6 +153,14 @@ pub enum AttrValue {
     CompressedFloatVec(CompressedData),
     /// Null/missing value - preserves the fact that an attribute is missing
     Null,
+    /// Reference to a subgraph stored in GraphPool (for hierarchical meta-nodes)
+    SubgraphRef(SubgraphId),
+    /// Array of node IDs (for storing node collections as attributes)
+    NodeArray(Vec<NodeId>),
+    /// Array of edge IDs (for storing edge collections as attributes)
+    EdgeArray(Vec<EdgeId>),
+    /// JSON-serialized complex data (lists, dicts, nested structures)
+    Json(String),
 }
 
 /// Custom PartialEq implementation that compares logical content across storage variants
@@ -103,7 +175,11 @@ impl PartialEq for AttrValue {
             (Bool(a), Bool(b)) => a == b,
             (SmallInt(a), SmallInt(b)) => a == b,
             (FloatVec(a), FloatVec(b)) => a == b,
+            (IntVec(a), IntVec(b)) => a == b,
+            (TextVec(a), TextVec(b)) => a == b,
+            (BoolVec(a), BoolVec(b)) => a == b,
             (Bytes(a), Bytes(b)) => a == b,
+            (Json(a), Json(b)) => a == b,
 
             // Cross-type integer comparisons
             (Int(a), SmallInt(b)) => *a == *b as i64,
@@ -151,20 +227,19 @@ impl PartialEq for AttrValue {
             }
 
             // Cross-type float vector comparisons
-            (FloatVec(a), CompressedFloatVec(b)) => {
-                // TODO: Implement compressed float vector decompression when needed
-                let _ = (a, b);
-                false
-            }
-            (CompressedFloatVec(a), FloatVec(b)) => {
-                // TODO: Implement compressed float vector decompression when needed
-                let _ = (a, b);
-                false
-            }
+            (FloatVec(a), CompressedFloatVec(b)) => match b.decompress_float_vec() {
+                Ok(decompressed_b) => a == &decompressed_b,
+                Err(_) => false,
+            },
+            (CompressedFloatVec(a), FloatVec(b)) => match a.decompress_float_vec() {
+                Ok(decompressed_a) => &decompressed_a == b,
+                Err(_) => false,
+            },
             (CompressedFloatVec(a), CompressedFloatVec(b)) => {
-                // TODO: Implement compressed float vector comparison when needed
-                let _ = (a, b);
-                false
+                match (a.decompress_float_vec(), b.decompress_float_vec()) {
+                    (Ok(decompressed_a), Ok(decompressed_b)) => decompressed_a == decompressed_b,
+                    _ => false,
+                }
             }
 
             // Null comparisons - Null only equals Null
@@ -178,7 +253,7 @@ impl PartialEq for AttrValue {
 
 /// Memory-efficient string storage that avoids heap allocation for short strings
 /// MEMORY OPTIMIZATION: Stores strings <= 15 bytes inline, larger ones on heap
-#[derive(Debug, Clone, PartialEq, serde::Serialize)]
+#[derive(Debug, Clone, PartialEq, serde::Serialize, serde::Deserialize)]
 pub enum CompactString {
     /// Inline storage for strings up to 15 bytes (fits in 16 bytes total)
     Inline { data: [u8; 15], len: u8 },
@@ -247,36 +322,64 @@ impl Hash for AttrValue {
                     f.to_bits().hash(state); // Hash each float using to_bits()
                 }
             }
+            AttrValue::IntVec(v) => {
+                4u8.hash(state); // Discriminant for IntVec variant
+                v.hash(state);
+            }
+            AttrValue::TextVec(v) => {
+                5u8.hash(state); // Discriminant for TextVec variant
+                v.hash(state);
+            }
+            AttrValue::BoolVec(v) => {
+                6u8.hash(state); // Discriminant for BoolVec variant
+                v.hash(state);
+            }
             AttrValue::Bool(b) => {
-                4u8.hash(state); // Discriminant for Bool variant
+                7u8.hash(state); // Discriminant for Bool variant
                 b.hash(state);
             }
             AttrValue::CompactText(cs) => {
-                5u8.hash(state); // Discriminant for CompactText variant
+                8u8.hash(state); // Discriminant for CompactText variant
                 cs.as_str().hash(state);
             }
             AttrValue::SmallInt(i) => {
-                6u8.hash(state); // Discriminant for SmallInt variant
+                9u8.hash(state); // Discriminant for SmallInt variant
                 i.hash(state);
             }
             AttrValue::Bytes(bytes) => {
-                7u8.hash(state); // Discriminant for Bytes variant
+                10u8.hash(state); // Discriminant for Bytes variant
                 bytes.hash(state);
             }
             AttrValue::CompressedText(cd) => {
-                8u8.hash(state); // Discriminant for CompressedText variant
+                11u8.hash(state); // Discriminant for CompressedText variant
                 cd.data.hash(state);
                 cd.original_size.hash(state);
             }
             AttrValue::CompressedFloatVec(cd) => {
-                9u8.hash(state); // Discriminant for CompressedFloatVec variant
+                12u8.hash(state); // Discriminant for CompressedFloatVec variant
                 cd.data.hash(state);
                 cd.original_size.hash(state);
             }
             AttrValue::Null => {
-                10u8.hash(state); // Discriminant for Null variant
+                13u8.hash(state); // Discriminant for Null variant
                                   // No additional data to hash for Null
             }
+            AttrValue::SubgraphRef(id) => {
+                14u8.hash(state); // Discriminant for SubgraphRef variant
+                id.hash(state);
+            }
+            AttrValue::NodeArray(nodes) => {
+                15u8.hash(state); // Discriminant for NodeArray variant
+                nodes.hash(state);
+            }
+            AttrValue::EdgeArray(edges) => {
+                16u8.hash(state); // Discriminant for EdgeArray variant
+                edges.hash(state);
+            }
+            AttrValue::Json(json_str) => {
+                17u8.hash(state); // Discriminant for Json variant
+                json_str.hash(state);
+            }
         }
     }
 }
@@ -284,6 +387,12 @@ impl Hash for AttrValue {
 // Manual Eq implementation to handle f32 comparison
 impl Eq for AttrValue {}
 
+impl Default for AttrValue {
+    fn default() -> Self {
+        AttrValue::Null
+    }
+}
+
 impl PartialOrd for AttrValue {
     fn partial_cmp(&self, other: &Self) -> Option<std::cmp::Ordering> {
         Some(self.cmp(other))
@@ -301,8 +410,8 @@ impl Ord for AttrValue {
             (SmallInt(a), SmallInt(b)) => a.cmp(b),
             (Float(a), Float(b)) => a.partial_cmp(b).unwrap_or(Ordering::Equal),
             (Bool(a), Bool(b)) => a.cmp(b),
-            (Text(a), Text(b)) => a.cmp(b),
-            (CompactText(a), CompactText(b)) => a.as_str().cmp(b.as_str()),
+            (Text(a), Text(b)) => AttrValue::cmp_text_content(a, b),
+            (CompactText(a), CompactText(b)) => AttrValue::cmp_text_content(a.as_str(), b.as_str()),
             (FloatVec(a), FloatVec(b)) => a.len().cmp(&b.len()).then_with(|| {
                 // Compare vectors lexicographically
                 for (va, vb) in a.iter().zip(b.iter()) {
@@ -314,7 +423,12 @@ impl Ord for AttrValue {
                 Ordering::Equal
             }),
             (Bytes(a), Bytes(b)) => a.cmp(b),
-            (CompressedText(a), CompressedText(b)) => a.data.cmp(&b.data),
+            (CompressedText(a), CompressedText(b)) => {
+                match (a.decompress_text(), b.decompress_text()) {
+                    (Ok(lhs), Ok(rhs)) => AttrValue::cmp_text_content(lhs.as_str(), rhs.as_str()),
+                    _ => a.data.cmp(&b.data),
+                }
+            }
             (CompressedFloatVec(a), CompressedFloatVec(b)) => a.data.cmp(&b.data),
 
             // Cross-type comparisons - order by type discriminant first
@@ -322,8 +436,24 @@ impl Ord for AttrValue {
             (SmallInt(a), Int(b)) => (*a as i64).cmp(b),
 
             // Text comparisons across storage types
-            (Text(a), CompactText(b)) => a.as_str().cmp(b.as_str()),
-            (CompactText(a), Text(b)) => a.as_str().cmp(b.as_str()),
+            (Text(a), CompactText(b)) => AttrValue::cmp_text_content(a.as_str(), b.as_str()),
+            (CompactText(a), Text(b)) => AttrValue::cmp_text_content(a.as_str(), b.as_str()),
+            (Text(a), CompressedText(b)) => match b.decompress_text() {
+                Ok(text) => AttrValue::cmp_text_content(a.as_str(), text.as_str()),
+                Err(_) => std::cmp::Ordering::Equal,
+            },
+            (CompressedText(a), Text(b)) => match a.decompress_text() {
+                Ok(text) => AttrValue::cmp_text_content(text.as_str(), b.as_str()),
+                Err(_) => std::cmp::Ordering::Equal,
+            },
+            (CompactText(a), CompressedText(b)) => match b.decompress_text() {
+                Ok(text) => AttrValue::cmp_text_content(a.as_str(), text.as_str()),
+                Err(_) => std::cmp::Ordering::Equal,
+            },
+            (CompressedText(a), CompactText(b)) => match a.decompress_text() {
+                Ok(text) => AttrValue::cmp_text_content(text.as_str(), b.as_str()),
+                Err(_) => std::cmp::Ordering::Equal,
+            },
 
             // Different types - order by discriminant (type priority)
             _ => self.type_discriminant().cmp(&other.type_discriminant()),
@@ -341,14 +471,217 @@ impl AttrValue {
             AttrValue::Bool(_) => 3,
             AttrValue::Text(_) | AttrValue::CompactText(_) | AttrValue::CompressedText(_) => 4,
             AttrValue::FloatVec(_) | AttrValue::CompressedFloatVec(_) => 5,
-            AttrValue::Bytes(_) => 6,
+            AttrValue::IntVec(_) => 6,
+            AttrValue::TextVec(_) => 7,
+            AttrValue::BoolVec(_) => 8,
+            AttrValue::Bytes(_) => 9,
+            AttrValue::SubgraphRef(_) => 10,
+            AttrValue::NodeArray(_) => 11,
+            AttrValue::EdgeArray(_) => 12,
+            AttrValue::Json(_) => 13,
+        }
+    }
+}
+
+impl AttrValue {
+    fn cmp_text_content(lhs: &str, rhs: &str) -> std::cmp::Ordering {
+        use std::cmp::Ordering;
+
+        fn parse_numeric(input: &str) -> Option<f64> {
+            let trimmed = input.trim();
+            if trimmed.is_empty() {
+                None
+            } else {
+                trimmed.parse::<f64>().ok()
+            }
+        }
+
+        match (parse_numeric(lhs), parse_numeric(rhs)) {
+            (Some(a), Some(b)) => a.partial_cmp(&b).unwrap_or(Ordering::Equal),
+            _ => lhs.cmp(rhs),
+        }
+    }
+}
+
+/*
+=== NUMERIC TYPE SYSTEM ===
+Type infrastructure for the advanced NumArray type system.
+This supports seamless conversions between different numeric representations
+and enables type-aware operations across BaseArray, NumArray, and Matrix.
+*/
+
+/// Numeric type classification for the advanced type system
+/// This enum represents the fundamental numeric types that can be stored
+/// in NumArrays and supports automatic type inference and conversion.
+#[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
+pub enum NumericType {
+    /// 32-bit signed integer
+    Int32,
+    /// 64-bit signed integer  
+    Int64,
+    /// 32-bit floating point
+    Float32,
+    /// 64-bit floating point
+    Float64,
+    /// Boolean values (can be treated as 0/1)
+    Bool,
+}
+
+impl NumericType {
+    /// Check if this type can be losslessly converted to another type
+    pub fn can_convert_to(&self, target: NumericType) -> bool {
+        use NumericType::*;
+        match (self, target) {
+            // Same types can always convert
+            (a, b) if *a == b => true,
+
+            // Integer promotions (lossless)
+            (Int32, Int64) => true,
+            (Int32, Float64) => true, // i32 can fit in f64 exactly
+            (Bool, Int32) | (Bool, Int64) => true,
+            (Bool, Float32) | (Bool, Float64) => true,
+
+            // Float promotions (lossless)
+            (Float32, Float64) => true,
+
+            // All other conversions are potentially lossy
+            _ => false,
+        }
+    }
+
+    /// Get the "highest" type that can represent both types without loss
+    pub fn promote_with(&self, other: NumericType) -> NumericType {
+        use NumericType::*;
+        match (*self, other) {
+            // Same types
+            (a, b) if a == b => a,
+
+            // Bool promotes to anything
+            (Bool, other) | (other, Bool) => other,
+
+            // Float64 is the ultimate promotion target
+            (Float64, _) | (_, Float64) => Float64,
+            (Float32, _) | (_, Float32) => Float32,
+
+            // Integer promotions
+            (Int64, _) | (_, Int64) => Int64,
+            // Note: (Int32, _) | (_, Int32) => Int32 is unreachable due to above patterns
+            // If we reach here, both types must be Int32 (caught by same types) or lower precision
+            _ => Int32, // Default fallback for any remaining cases
+        }
+    }
+
+    /// Get the size in bytes of this numeric type
+    pub fn size_bytes(&self) -> usize {
+        match self {
+            NumericType::Bool => 1,
+            NumericType::Int32 => 4,
+            NumericType::Float32 => 4,
+            NumericType::Int64 => 8,
+            NumericType::Float64 => 8,
+        }
+    }
+
+    /// Check if this type is an integer type
+    pub fn is_integer(&self) -> bool {
+        matches!(
+            self,
+            NumericType::Int32 | NumericType::Int64 | NumericType::Bool
+        )
+    }
+
+    /// Check if this type is a floating point type
+    pub fn is_float(&self) -> bool {
+        matches!(self, NumericType::Float32 | NumericType::Float64)
+    }
+}
+
+impl AttrValue {
+    /// Get the numeric type of this AttrValue, if it represents a numeric value
+    pub fn numeric_type(&self) -> Option<NumericType> {
+        match self {
+            AttrValue::Bool(_) => Some(NumericType::Bool),
+            AttrValue::SmallInt(_) => Some(NumericType::Int32),
+            AttrValue::Int(_) => Some(NumericType::Int64),
+            AttrValue::Float(_) => Some(NumericType::Float32),
+            _ => None,
+        }
+    }
+
+    /// Check if this AttrValue represents a numeric value
+    pub fn is_numeric(&self) -> bool {
+        self.numeric_type().is_some()
+    }
+
+    /// Convert this AttrValue to a specific numeric type, if possible
+    #[allow(dead_code)]
+    pub fn to_numeric_type(&self, target: NumericType) -> Option<AttrValue> {
+        match (self, target) {
+            // Bool conversions
+            (AttrValue::Bool(b), NumericType::Bool) => Some(AttrValue::Bool(*b)),
+            (AttrValue::Bool(b), NumericType::Int32) => {
+                Some(AttrValue::SmallInt(if *b { 1 } else { 0 }))
+            }
+            (AttrValue::Bool(b), NumericType::Int64) => {
+                Some(AttrValue::Int(if *b { 1 } else { 0 }))
+            }
+            (AttrValue::Bool(b), NumericType::Float32) => {
+                Some(AttrValue::Float(if *b { 1.0 } else { 0.0 }))
+            }
+            (AttrValue::Bool(b), NumericType::Float64) => {
+                // Note: We store f32 in AttrValue, so convert to f32
+                Some(AttrValue::Float(if *b { 1.0 } else { 0.0 }))
+            }
+
+            // SmallInt (i32) conversions
+            (AttrValue::SmallInt(i), NumericType::Bool) => Some(AttrValue::Bool(*i != 0)),
+            (AttrValue::SmallInt(i), NumericType::Int32) => Some(AttrValue::SmallInt(*i)),
+            (AttrValue::SmallInt(i), NumericType::Int64) => Some(AttrValue::Int(*i as i64)),
+            (AttrValue::SmallInt(i), NumericType::Float32) => Some(AttrValue::Float(*i as f32)),
+            (AttrValue::SmallInt(i), NumericType::Float64) => Some(AttrValue::Float(*i as f32)), // Still stored as f32
+
+            // Int (i64) conversions
+            (AttrValue::Int(i), NumericType::Bool) => Some(AttrValue::Bool(*i != 0)),
+            (AttrValue::Int(i), NumericType::Int32) => {
+                if *i >= i32::MIN as i64 && *i <= i32::MAX as i64 {
+                    Some(AttrValue::SmallInt(*i as i32))
+                } else {
+                    None // Overflow
+                }
+            }
+            (AttrValue::Int(i), NumericType::Int64) => Some(AttrValue::Int(*i)),
+            (AttrValue::Int(i), NumericType::Float32) => Some(AttrValue::Float(*i as f32)), // May lose precision
+            (AttrValue::Int(i), NumericType::Float64) => Some(AttrValue::Float(*i as f32)), // Still stored as f32
+
+            // Float (f32) conversions
+            (AttrValue::Float(f), NumericType::Bool) => Some(AttrValue::Bool(*f != 0.0)),
+            (AttrValue::Float(f), NumericType::Int32) => {
+                let rounded = f.round();
+                if rounded >= i32::MIN as f32 && rounded <= i32::MAX as f32 {
+                    Some(AttrValue::SmallInt(rounded as i32))
+                } else {
+                    None // Overflow
+                }
+            }
+            (AttrValue::Float(f), NumericType::Int64) => {
+                let rounded = f.round();
+                if rounded.is_finite() {
+                    Some(AttrValue::Int(rounded as i64))
+                } else {
+                    None // Infinity or NaN
+                }
+            }
+            (AttrValue::Float(f), NumericType::Float32) => Some(AttrValue::Float(*f)),
+            (AttrValue::Float(f), NumericType::Float64) => Some(AttrValue::Float(*f)), // Still stored as f32
+
+            _ => None, // Non-numeric input
         }
     }
 }
 
 /// Compressed data storage for large values (Memory Optimization 3)
 /// Uses simple run-length encoding and basic compression
-#[derive(Debug, Clone, PartialEq, serde::Serialize)]
+#[derive(Debug, Clone, PartialEq, serde::Serialize, serde::Deserialize)]
 pub struct CompressedData {
     /// Compressed bytes
     pub data: Vec<u8>,
@@ -358,7 +691,7 @@ pub struct CompressedData {
     algorithm: CompressionAlgorithm,
 }
 
-#[derive(Debug, Clone, PartialEq, serde::Serialize)]
+#[derive(Debug, Clone, PartialEq, serde::Serialize, serde::Deserialize)]
 pub enum CompressionAlgorithm {
     /// No compression (passthrough)
     None,
@@ -500,7 +833,7 @@ impl CompressedData {
 
     /// Decode run-length encoded data
     fn run_length_decode(input: &[u8]) -> Result<Vec<u8>, &'static str> {
-        if input.len() % 2 != 0 {
+        if !input.len().is_multiple_of(2) {
             return Err("Invalid run-length encoded data");
         }
 
@@ -545,6 +878,9 @@ pub enum AttrValueType {
     Int,
     Text,
     FloatVec,
+    IntVec,
+    TextVec,
+    BoolVec,
     Bool,
     CompactText,
     SmallInt,
@@ -552,6 +888,10 @@ pub enum AttrValueType {
     CompressedText,
     CompressedFloatVec,
     Null,
+    SubgraphRef,
+    NodeArray,
+    EdgeArray,
+    Json,
 }
 
 impl AttrValueType {
@@ -567,13 +907,22 @@ impl AttrValueType {
 impl std::fmt::Display for AttrValue {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match self {
-            AttrValue::Float(val) => write!(f, "{}", val),
+            AttrValue::Float(val) => {
+                if val.is_nan() {
+                    write!(f, "nan")
+                } else {
+                    write!(f, "{}", val)
+                }
+            }
             AttrValue::Int(val) => write!(f, "{}", val),
             AttrValue::Text(val) => write!(f, "{}", val),
             AttrValue::Bool(val) => write!(f, "{}", val),
             AttrValue::SmallInt(val) => write!(f, "{}", val),
             AttrValue::CompactText(val) => write!(f, "{}", val.as_str()),
             AttrValue::FloatVec(val) => write!(f, "{:?}", val),
+            AttrValue::IntVec(val) => write!(f, "{:?}", val),
+            AttrValue::TextVec(val) => write!(f, "{:?}", val),
+            AttrValue::BoolVec(val) => write!(f, "{:?}", val),
             AttrValue::Bytes(val) => write!(f, "{:?}", val),
             AttrValue::CompressedText(val) => match val.decompress_text() {
                 Ok(text) => write!(f, "{}", text),
@@ -583,7 +932,11 @@ impl std::fmt::Display for AttrValue {
                 Ok(vec) => write!(f, "{:?}", vec),
                 Err(_) => write!(f, "[compressed float vec]"),
             },
-            AttrValue::Null => write!(f, "NaN"),
+            AttrValue::Null => write!(f, "nan"),
+            AttrValue::SubgraphRef(id) => write!(f, "SubgraphRef({})", id),
+            AttrValue::NodeArray(nodes) => write!(f, "NodeArray({:?})", nodes),
+            AttrValue::EdgeArray(edges) => write!(f, "EdgeArray({:?})", edges),
+            AttrValue::Json(json) => write!(f, "Json({})", json),
         }
     }
 }
@@ -596,6 +949,9 @@ impl AttrValue {
             AttrValue::Int(_) => "Int",
             AttrValue::Text(_) => "Text",
             AttrValue::FloatVec(_) => "FloatVec",
+            AttrValue::IntVec(_) => "IntVec",
+            AttrValue::TextVec(_) => "TextVec",
+            AttrValue::BoolVec(_) => "BoolVec",
             AttrValue::Bool(_) => "Bool",
             AttrValue::CompactText(_) => "CompactText",
             AttrValue::SmallInt(_) => "SmallInt",
@@ -603,6 +959,10 @@ impl AttrValue {
             AttrValue::CompressedText(_) => "CompressedText",
             AttrValue::CompressedFloatVec(_) => "CompressedFloatVec",
             AttrValue::Null => "Null",
+            AttrValue::SubgraphRef(_) => "SubgraphRef",
+            AttrValue::NodeArray(_) => "NodeArray",
+            AttrValue::EdgeArray(_) => "EdgeArray",
+            AttrValue::Json(_) => "Json",
         }
     }
 
@@ -613,6 +973,9 @@ impl AttrValue {
             AttrValue::Int(_) => AttrValueType::Int,
             AttrValue::Text(_) => AttrValueType::Text,
             AttrValue::FloatVec(_) => AttrValueType::FloatVec,
+            AttrValue::IntVec(_) => AttrValueType::IntVec,
+            AttrValue::TextVec(_) => AttrValueType::TextVec,
+            AttrValue::BoolVec(_) => AttrValueType::BoolVec,
             AttrValue::Bool(_) => AttrValueType::Bool,
             AttrValue::CompactText(_) => AttrValueType::CompactText,
             AttrValue::SmallInt(_) => AttrValueType::SmallInt,
@@ -620,6 +983,10 @@ impl AttrValue {
             AttrValue::CompressedText(_) => AttrValueType::CompressedText,
             AttrValue::CompressedFloatVec(_) => AttrValueType::CompressedFloatVec,
             AttrValue::Null => AttrValueType::Null,
+            AttrValue::SubgraphRef(_) => AttrValueType::SubgraphRef,
+            AttrValue::NodeArray(_) => AttrValueType::NodeArray,
+            AttrValue::EdgeArray(_) => AttrValueType::EdgeArray,
+            AttrValue::Json(_) => AttrValueType::Json,
         }
     }
 
@@ -632,6 +999,18 @@ impl AttrValue {
             AttrValue::FloatVec(v) => {
                 std::mem::size_of::<Vec<f32>>() + v.capacity() * std::mem::size_of::<f32>()
             }
+            AttrValue::IntVec(v) => {
+                std::mem::size_of::<Vec<i64>>() + v.capacity() * std::mem::size_of::<i64>()
+            }
+            AttrValue::TextVec(v) => {
+                std::mem::size_of::<Vec<String>>()
+                    + v.iter()
+                        .map(|s| std::mem::size_of::<String>() + s.capacity())
+                        .sum::<usize>()
+            }
+            AttrValue::BoolVec(v) => {
+                std::mem::size_of::<Vec<bool>>() + v.capacity() * std::mem::size_of::<bool>()
+            }
             AttrValue::Bool(_) => std::mem::size_of::<bool>(),
             AttrValue::CompactText(cs) => cs.memory_size(),
             AttrValue::SmallInt(_) => std::mem::size_of::<i32>(),
@@ -639,6 +1018,16 @@ impl AttrValue {
             AttrValue::CompressedText(cd) => cd.memory_size(),
             AttrValue::CompressedFloatVec(cd) => cd.memory_size(),
             AttrValue::Null => 0, // Null values take no memory
+            AttrValue::SubgraphRef(_) => std::mem::size_of::<crate::types::SubgraphId>(),
+            AttrValue::NodeArray(nodes) => {
+                std::mem::size_of::<Vec<crate::types::NodeId>>()
+                    + nodes.capacity() * std::mem::size_of::<crate::types::NodeId>()
+            }
+            AttrValue::EdgeArray(edges) => {
+                std::mem::size_of::<Vec<crate::types::EdgeId>>()
+                    + edges.capacity() * std::mem::size_of::<crate::types::EdgeId>()
+            }
+            AttrValue::Json(s) => std::mem::size_of::<String>() + s.capacity(),
         }
     }
 
@@ -774,7 +1163,7 @@ pub struct MemoryEfficiency {
 }
 
 /// Data compression statistics
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, serde::Serialize, serde::Deserialize)]
 pub struct CompressionStatistics {
     /// Number of compressed attributes
     pub compressed_attributes: usize,
